@@ -90,7 +90,7 @@ file.
 
 =head1 DESCRIPTION
 
-StreamFinder::Youtube accepts a valid full YouTube video ID or URL on 
+StreamFinder::Youtube accepts a valid full YouTube video ID, or page URL on 
 youtube, et. al. that the "youtube-dl" program supports, 
 and returns the actual stream URL, title, and cover art icon for that video.  
 The purpose is that one needs this URL in order to have the option to 
@@ -103,6 +103,13 @@ open-source "audacious" audio player).  "fauxdacious" incorporates this
 module to decode and play youtube.com videos.  This is a submodule of the 
 general StreamFinder module.
 
+NOTE:  This module may return either Youtube or non-Youtube videos and streams 
+for non-Youtube sites, including videos embedded in IFRAME tags and even 
+Rumble.com videos found in some non-Youtube sites (L<StreamFinder::Rumble> 
+required).  See the I<-noiframes> and I<-youtubeonly> flags below for limiting 
+this feature.  Also note:  these videos, etc. are handled here and not 
+by L<StreamFinder::Anystream>.
+
 Depends:  
 
 L<URI::Escape>, L<HTML::Entities>, L<LWP::UserAgent>, 
@@ -117,11 +124,12 @@ such as yt-dlp.
 [, I<-secure> [ => 0|1 ]] 
 [, I<-fast> [ => 0|1 ]] 
 [, I<-format> => "youtube-dl format specification" ] 
-[, I<-formatonly> [ => 0|1 ]] [, I<-noiframes> [ => 0|1 ]]
-[, I<-user-agent> => "user-agent string"]
-[, I<-userid> => "youtube-user-id", I<-userpw> => "password"]
-[, I<-youtube-dl> => "youtube-dl program"]
-[, I<-youtube-dl-args> => "youtube-dl arguments"]
+[, I<-formatonly> [ => 0|1 ]] [, I<-noiframes> [ => 0|1 ]] 
+[, I<-youtubeonly> [ => 0|1 ]] 
+[, I<-user-agent> => "user-agent string"] 
+[, I<-userid> => "youtube-user-id", I<-userpw> => "password"] 
+[, I<-youtube-dl> => "youtube-dl program"] 
+[, I<-youtube-dl-args> => "youtube-dl arguments"] 
 [, I<-youtube-dl-add-args> => "youtube-dl additional arguments"])
 
 Accepts a youtube.com video ID, or any full URL that youtube-dl supports 
@@ -149,7 +157,7 @@ if you know the video is NOT a YouTube video or you don't care about
 the artist (youtube channel's owner), artist icon, fields, etc.  
 Default is 0 (false / unset).
 
-If I<-noiframes>, if set to 1 (I<true>) means only 
+If I<-noiframes> is specified (set to 1 (true)), then only 
 process actual video URLs, not search the page for an iframe containing 
 a video URL (a new feature with v0.47).  This is used primarily internally 
 to prevent possible recursion when StreamFinder::YouTube finds an iframe 
@@ -159,6 +167,17 @@ again on that URL to find the stream).  Default is 0 (false / unset) - search
 for StreamFinder-searchable URLs in an iframe, if the page is HTML and not an 
 actual video URL.
 
+If I<-youtubeonly> Some non-Youtube pages have embedded Rumble (Rumble.com) 
+videos embedded in them and since StreamFinder::Youtube is somewhat of a 
+"catchall" (for videos), and we (the Author) prefer the less-woke Rumble to 
+Youtube (whish has major censorship issues), we search for embedded Rumble 
+videos here, as opposed to L<StreamFinder::Rumble> or 
+L<StreamFinder::AnyStream>, and, upon finding one, we return that rather than 
+continuing the search for Youtube videos.  To NOT do this (consider only 
+embedded Youtube videos), specify this / set it to 1 (true).  
+Default 0 (false) - accept Rumble (or other non-Youtube) videos, if found 
+first.  NOTE:  This option is effectively set (true) if I<-noiframes> is set!
+
 The optional I<-secure> argument can be either 0 or 1 (I<false> or I<true>).  
 If 1 then only secure ("https://") streams will be returned.  Default for 
 I<-secure> is 0 (false) - return all streams (http and https).
@@ -438,12 +457,17 @@ sub new
 	return undef  unless ($url);
 
 	my $self = $class->SUPER::new('Youtube', @_);
+
+	#SET DEFAULTS FOR FLAGS:
 	$DEBUG = $self->{'debug'}  if (defined $self->{'debug'});
+	$self->{'formatonly'} = 0  unless (defined $self->{'formatonly'});
 	$self->{'noiframes'} = 0  unless (defined $self->{'noiframes'});
+	$self->{'youtubeonly'} = 0  unless (defined $self->{'youtubeonly'});
 	#DEFAULT YOUTUBE-DL ARGUMENTS:
 	$self->{'youtube-dl-args'} = '--get-url --get-format --get-thumbnail --get-title --get-description --get-id'
 			unless (defined $self->{'youtube-dl-args'});
 
+	#FETCH ANY PARAMETERS PASSED TO THE new() FUNCTION (OVERRIDE ANY SET IN _class (GENERAL OR CONFIG FILES)):
 	while (@_) {
 		if ($_[0] =~ /^\-?fast$/o) {
 			shift;
@@ -451,6 +475,9 @@ sub new
 		} elsif ($_[0] =~ /^\-?noiframes$/o) {
 			shift;
 			$self->{'noiframes'} = (defined $_[0]) ? shift : 1;
+		} elsif ($_[0] =~ /^\-?youtubeonly$/o) {
+			shift;
+			$self->{'youtubeonly'} = (defined $_[0]) ? shift : 1;
 		} elsif ($_[0] =~ /^\-?formatonly$/o) {
 			shift;
 			$self->{'formatonly'} = (defined $_[0]) ? shift : 1;
@@ -470,30 +497,90 @@ sub new
 			shift;
 			$self->{'youtube-dl-add-args'} = shift  if (defined $_[0]);
 		} else {
-			shift;
+			shift;  #DISCARD ANY OTHERS.
 		}
 	}
+	$self->{'youtubeonly'} = 1  if ($self->{'noiframes'});  #NO EMBEDDED RUMBLE-SEARCH IF NO IFRAMES ALLOWED!
 
 	$self->{'youtube-dl'} = 'youtube-dl'  unless (defined $self->{'youtube-dl'});
 
 	print STDERR "-0(Youtube): URL=$url=\n"  if ($DEBUG);
 	$url =~ s/\?autoplay\=true$//;  #STRIP THIS OFF SO WE DON'T HAVE TO.
 	(my $url2fetch = $url);
+	$self->{'_isaYtPage'} = 1;
 	#DEPRECIATED (STATION-IDS NOW INCLUDE STUFF BEFORE THE DASH: ($self->{'id'} = $url) =~ s#^.*\-([a-z]\d+)\/?$#$1#;
 	if ($url2fetch =~ m#^https?\:#) {
+		$self->{'_isaYtPage'} = 0  unless ($url2fetch =~ /\b(?:youtube\.|youtu.be|ytimg\.)\b/);
 #$url2fetch =~ s/www\.youtube\.com/youtube\.be/;  #WWW.YOUTUBE.COM SEEMS TO NOW BE BLOCKING youtube-dl?! :/
 		$self->{'id'} = $1  if ($url2fetch =~ m#\/([^\/]+)\/?$#);
 		$self->{'id'} =~ s/^watch\?v\=//;
 		$self->{'id'} =~ s/[\?\&].*$//;
+		$self->{'id'} = $1  if (!$self->{'_isaYtPage'} && $url2fetch =~ m#id[\=\:\#]?([^\/\s\=\:\#]+)#);
 	} else {
 		$self->{'id'} = $url;
 		$url2fetch = 'https://youtube.be/watch?v=' . $url;
 	}
-	print STDERR "-1 FETCHING URL=$url= VIA youtube-dl: ID=".$self->{'id'}."=\n"  if ($DEBUG);
+	print STDERR "-1 (isYT=".$self->{'_isaYtPage'}.") FETCHING URL=$url= VIA youtube-dl: ID=".$self->{'id'}."=\n"  if ($DEBUG);
 	$self->{'genre'} = 'Video';
 	$self->{'albumartist'} = $url2fetch;
 
-	#FIRST:  GET STREAMS, THUMBNAIL, ETC. FROM youtube-dl:
+	#FIRST:  IF NON-YOUTUBE PAGE, LOOK FOR ANYTHING EMBEDDED IN AN IFRAME:
+
+	unless ($self->{'_isaYtPage'} || $self->{'noiframes'}) {
+		print STDERR "..1a:See if we have a StreamFinder-supported URL in 1st iframe?...\n"  if ($DEBUG);
+		my $embedded_video;
+		my $html = '';
+		my $ua = LWP::UserAgent->new(@{$self->{'_userAgentOps'}});		
+		$ua->timeout($self->{'timeout'});
+		$ua->max_size(1024);  #LIMIT FETCH-SIZE TO AVOID INFINITELY DOWNLOADING A STREAM!
+		$ua->cookie_jar({});
+		$ua->env_proxy;
+	 	my $response = $ua->get($url);
+	 	$html = $response->decoded_content  if ($response->is_success);
+	 	if ($html =~ /\<\!DOCTYPE\s+(?:html|text)/i) {  #IF WE'RE AN HTML DOC. (NOT A STREAM!), THEN FETCH THE WHOLE THING:
+			$ua->max_size(undef);  #(NOW OK TO FETCH THE WHOLE DOCUMENT)
+		 	my $response = $ua->get($url);
+		 	$html = $response->decoded_content  if ($response->is_success);
+			while ($html && $html =~ s#\<iframe([^\>]+)\>##s) {
+				my $one = $1;
+				my $embeddedURL = ($one =~ m#\"(https?\:\/\/[^\"]+)#s) ? $1 : '';
+				print STDERR "--embedded IFRAME url=$embeddedURL=\n"  if ($DEBUG);
+				if ($embeddedURL) {
+					my $haveStreamFinder = 0;
+					eval { require 'StreamFinder.pm'; $haveStreamFinder = 1; };
+					if ($haveStreamFinder) {
+						my %globalArgs = (-noiframes => 1, -debug => $DEBUG);
+						foreach my $arg (qw(log logfmt)) {
+							$globalArgs{$arg} = $self->{$arg}  if (defined($self->{$arg}) && $self->{$arg});
+						}
+						$embedded_video = new StreamFinder($embeddedURL, %globalArgs);
+					}
+					last;
+				}
+			}
+			return $embedded_video  if (defined($embedded_video) && $embedded_video->count() > 0);
+
+			unless ($self->{'youtubeonly'}) {
+				if ($html =~ /\bRumble\s*\(\"play\"\,\s+\{\"video\"\:\"([a-z0-9\-\_]+)\"/si) {
+					#EXTRACT CERTAIN EMBEDDED RUMBLE VIDEOS NOT NECESSARILY IN AN IFRAME:
+					my $embeddedURL = 'https://rumble.com/embed/' . $1;
+					my $haveRumble = 0;
+					print STDERR "---FOUND AN EMBEDDED RUMBLE VIDEO ($embeddedURL), SEE IF WE CAN GO WITH THAT!\n"  if ($DEBUG);
+					eval { require 'StreamFinder/Rumble.pm'; $haveRumble = 1; };
+					if ($haveRumble) {
+						my %globalArgs = (-debug => $DEBUG);
+						foreach my $arg (qw(log logfmt)) {
+							$globalArgs{$arg} = $self->{$arg}  if (defined($self->{$arg}) && $self->{$arg});
+						}
+						$embedded_video = new StreamFinder::Rumble($embeddedURL, %globalArgs);
+						return $embedded_video  if (defined($embedded_video) && $embedded_video->count() > 0);
+					}
+				}
+			}
+		}
+	}
+
+	#NEXT:  GET STREAMS, THUMBNAIL, ETC. FROM youtube-dl:
 
 	my $ytformat = (defined $self->{'format'}) ? $self->{'format'} : 'mp4';
 	my $ua = (defined $self->{'user-agent'}) ? (' --user-agent "'.$self->{'user-agent'}.'"') : '';
@@ -515,74 +602,29 @@ RETRYIT:
 	} else {
 		$cmd = $self->{'youtube-dl'} . " $ytdlArgs " . '"' . $url . '"';
 	}
-	print STDERR "--TRY($try of 1): youtube-dl: ARGS=$ytdlArgs= FMT=$ytformat= YT COMMAND===>$cmd<===\n"  if ($DEBUG);
+	print STDERR "--TRY($try of 1): youtube-dl: ARGS=$ytdlArgs= FMT=$ytformat= YT COMMAND==>$cmd<==\n"  if ($DEBUG);
 	$_ = `$cmd`;
 	print STDERR "--YT RETURNED DATA===>$_<===\n"  if ($DEBUG);
 	@ytdldata = split /\r?\n/s;
 	unless ($try || scalar(@ytdldata) > 0) {  #IF NOTHING FOUND, RETRY WITHOUT THE SPECIFIC FILE-FORMAT:
-		unless ($self->{'noiframes'}) {
-			print STDERR "..1a:See if we have a StreamFinder-supported URL in 1st iframe?...\n"  if ($DEBUG);
-			my $embedded_video;
-			my $html = '';
-			my $ua = LWP::UserAgent->new(@{$self->{'_userAgentOps'}});		
-			$ua->timeout($self->{'timeout'});
-			$ua->max_size(1024);  #LIMIT FETCH-SIZE TO AVOID INFINITELY DOWNLOADING A STREAM!
-			$ua->cookie_jar({});
-			$ua->env_proxy;
-		 	my $response = $ua->get($url);
-		 	$html = $response->decoded_content  if ($response->is_success);
-		 	if ($html =~ /\<\!DOCTYPE\s+(?:html|text)/i) {  #IF WE'RE AN HTML DOC. (NOT A STREAM!), THEN FETCH THE WHOLE THING:
-				$ua->max_size(undef);  #(NOW OK TO FETCH THE WHOLE DOCUMENT)
-			 	my $response = $ua->get($url);
-			 	$html = $response->decoded_content  if ($response->is_success);
-				while ($html && $html =~ s#\<iframe([^\>]+)\>##s) {
-					my $one = $1;
-					my $embeddedURL = ($one =~ m#\"(https?\:\/\/[^\"]+)#s) ? $1 : '';
-					print STDERR "--embedded IFRAME url=$embeddedURL=\n"  if ($DEBUG);
-					if ($embeddedURL) {
-						my $haveStreamFinder = 0;
-						eval { require 'StreamFinder.pm'; $haveStreamFinder = 1; };
-						if ($haveStreamFinder) {
-							my %globalArgs = (-noiframes => 1, -debug => $DEBUG);
-							foreach my $arg (qw(log logfmt)) {
-								$globalArgs{$arg} = $self->{$arg}  if (defined($self->{$arg}) && $self->{$arg});
-							}
-							$embedded_video = new StreamFinder($embeddedURL, %globalArgs);
-						}
-						last;
-					}
-				}
-				return $embedded_video  if (defined($embedded_video) && $embedded_video->count() > 0);
-				if ($html =~ /\bRumble\s*\(\"play\"\,\s+\{\"video\"\:\"([a-z0-9\-\_]+)\"/si) {
-					#EXTRACT CERTAIN EMBEDDED RUMBLE VIDEOS NOT NECESSARILY IN AN IFRAME:
-					my $embeddedURL = 'https://rumble.com/embed/' . $1;
-					my $haveStreamFinder = 0;
-					eval { require 'StreamFinder.pm'; $haveStreamFinder = 1; };
-					if ($haveStreamFinder) {
-						my %globalArgs = (-noiframes => 1, -debug => $DEBUG);
-						foreach my $arg (qw(log logfmt)) {
-							$globalArgs{$arg} = $self->{$arg}  if (defined($self->{$arg}) && $self->{$arg});
-						}
-						$embedded_video = new StreamFinder($embeddedURL, %globalArgs);
-						return $embedded_video  if (defined($embedded_video) && $embedded_video->count() > 0);
-					}
-				}
-			}
-		}
-		print STDERR "..1:No streams matching ".$ytformat." found, try again for any (audio, etc.)...\n"  if ($DEBUG);
 		$try++;
-		unless (defined($self->{'formatonly'}) && $self->{'formatonly'}) {
-			goto RETRYIT  if ($ytdlArgs =~ s/\-f\s+\"([^\"]+)\"//);
+		unless ($self->{'formatonly'}) {
+			print STDERR "..1:No ($ytformat) streams found, try again for any (audio, etc.)...\n"  if ($DEBUG);
+			goto RETRYIT  if ($ytdlArgs =~ s/\-f\s+\"[^\"]+\"//);
 		}
 	}
 	return undef unless (scalar(@ytdldata) > 0);
 
-	#NOTE:  ytdldata is ORDERED:  TITLE?, STREAM-URLS, THEN THE ICON URL, THEN DESCRIPTION, LASTLY FORMAT#!:
+	#NOTE:  ytdldata is ORDERED:  TITLE?, ID, STREAM-URLS, THEN THE ICON URL, THEN DESCRIPTION, LASTLY FORMATS!:
 	unless ($ytdldata[0] =~ m#^https?\:\/\/#) {
 		$_ = shift(@ytdldata);
 		$self->{'title'} ||= $_;
 	}
-	$self->{'_ytID'} = shift(@ytdldata)  if ($ytdlArgs =~ /\-\-get\-id\b/ && $ytdldata[0] =~ /^[a-z0-9]{11}$/i);
+	$self->{'_ytID'} = '';
+	if ($ytdlArgs =~ /\-\-get\-id\b/ && $ytdldata[0] !~ /^https?\:/) {  #SHOULD HAVE AN "ID":
+		my $get_id = shift(@ytdldata);
+		$self->{'_ytID'} = $get_id  if ($get_id =~ /^[a-z0-9\-\_]{11}$/i);
+	}
 	my $fmtline = ($ytdldata[$#ytdldata] =~ m#^https?\:\/\/#) ? '-none' : pop(@ytdldata);  #LAST LINE IS (USUALLY) THE LIST OF FORMATS RETURNED.
 	my @fmtsfound = split(/\+/, $fmtline);
 	$self->{'description'} = '';
@@ -603,19 +645,22 @@ RETRYIT:
 	$self->{'cnt'} = scalar @{$self->{'streams'}};
 	print STDERR "-STREAM COUNT=".$self->{'cnt'}."= FMTS=".join('|',@fmtsfound)."= ICON=".$self->{'iconurl'}."=\n"  if ($DEBUG);
 	unless ($try || $self->{'cnt'} > 0) {  #IF NO STREAMS FOUND, RETRY WITHOUT THE SPECIFIC FILE-FORMAT:
-		print STDERR "..2:No ($ytformat) streams found, try again for any (audio, etc.)...\n"  if ($DEBUG);
 		$try++;
-		$ytdlArgs =~ s/\-f\s+\"([^\"]+)\"//;
-		goto RETRYIT  if ($1);
+		unless (defined($self->{'formatonly'}) && $self->{'formatonly'}) {
+			print STDERR "..2:No ($ytformat) streams found, try again for any (audio, etc.)...\n"  if ($DEBUG);
+			goto RETRYIT  if ($ytdlArgs =~ s/\-f\s+\"[^\"]+\"//);
+		}
 	}
 
-	#NOW MANUALLY SCAN YOUTUBE PAGE TO TRY TO GET artist, description, year, ETC. DIRECTLY FROM PAGE (IF A YOUTUBE SITE):
+	#NOW MANUALLY SCRAPE YOUTUBE PAGE TO TRY TO GET artist, description, year, ETC. DIRECTLY FROM PAGE (IF A YOUTUBE SITE):
 
-	unless ($self->{'fast'}) {   #(FAST MEANS SKIP SCRAPING YOUTUBE PAGE FOR ADDTL. METADATA)
+	unless ($self->{'fast'}) {  #(FAST MEANS SKIP SCRAPING YOUTUBE PAGE FOR ADDTL. METADATA)
 		$try = 0;
 RETRYPAGE:
 		print STDERR "----(try2=$try= FETCHURL=$url2fetch=\n"  if ($DEBUG);
-		if ($url2fetch =~ /\b(?:youtube\.|youtu.be|ytimg\.)\b/) {  #WE'RE A YOUTUBE PAGE, FETCH METADATA:
+		if ($self->{'_isaYtPage'}) {  #WE'RE A YOUTUBE PAGE, FETCH METADATA:
+			#CONVERT "embedded" YT PAGES TO ACTUAL PAGE (EMBEDDED PAGES DON'T HAVE THE METADATA WE'RE SEEKING!:
+			$url2fetch = 'https://youtube.be/watch?v=' .$1  if ($url2fetch =~ m#\/embed\/([a-z0-9\-\_]{11})#i);
 			print STDERR "-2 (TRY=$try) FETCHING SCREEN URL=$url2fetch= ID=".$self->{'id'}."=\n"  if ($DEBUG);
 			my $ua = LWP::UserAgent->new(@{$self->{'_userAgentOps'}});		
 			$ua->timeout($self->{'timeout'});
@@ -653,14 +698,14 @@ RETRYPAGE:
 			$self->{'articonurl'} = $1  if ($html =~ m#(?:\"CHANNEL\"\,\"image\"|\"videoOwnerRenderer\")\:\{\"thumbnails"\:\[\{\"url\"\:\"([^\"]+)#s);
 			print  "--YT:2 CHANNEL ICON URL1=".$self->{'articonurl'}."=\n"  if ($DEBUG);
 			unless ($self->{'articonurl'}) {
-				my $ownerstuff = $1  if ($html =~ m#\"videoOwnerRenderer\"\:\{([^\}]+)#s);
+				my $ownerstuff = ($html =~ m#\"videoOwnerRenderer\"\:\{([^\}]+)#s) ? $1 : '';
 				$self->{'articonurl'} = $1  if ($ownerstuff =~ /\"url\"\:\"([^\"]+)/);
 			}
 			print  "--YT:2 CHANNEL ICON URL2=".$self->{'articonurl'}."=\n"  if ($DEBUG);
-		} elsif (!$try && defined($self->{'_ytID'}) && $self->{'_ytID'}) {
+		} elsif (!$try && $self->{'_ytID'}) { #WE'RE NOT A YOUTUBE PAGE, BUT WE HAVE THE YT ID, SO TRY TO FETCH IT FOR METADATA:
 			print STDERR "--WE ARE NOT A YT PAGE, BUT ytID=".$self->{'_ytID'}."= SO WE WILL TRY AGAIN!\n"  if ($DEBUG);
 			++$try;
-			#WE'RE NOT A YOUTUBE PAGE, BUT WE HAVE THE YT ID, SO TRY TO FETCH IT FOR METADATA:
+			++$self->{'_isaYtPage'};
 			$url2fetch = 'https://youtube.be/watch?v=' . $self->{'_ytID'};
 			goto RETRYPAGE;
 		}
